//******************************************************************************
// This unit maintains configuration information in the B1401.INI file, and
// provides the screen to allow the user to display and alter this information.
//******************************************************************************
unit BFOOCnfg;

interface

uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, StdCtrls, IniFiles,
     IBMDefs,      // Global, utility, and miscellaneous definitions
     BFOODefs;     // Global, utility, and miscellaneous definitions

type
  TConfigForm = class(TForm)
    PBOK: TButton;
    PBCancel: TButton;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    RBCS1: TRadioButton;
    RBCS4: TRadioButton;
    RBCS5: TRadioButton;
    RBCS6: TRadioButton;
    RBCS2: TRadioButton;
    RBCS3: TRadioButton;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    EBPOSName: TEdit;
    CBCoreInitCheck: TCheckBox;
    PBBrowse: TButton;
    OpenDialog: TOpenDialog;
    procedure PBOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RBCSClick(Sender: TObject);
    procedure PBBrowseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    IniFile:  TIniFile;
    CoreSize: Word;                // Temp value for core size
  public
//  PowerOn:  Boolean;             // Run "PowerOn.Ocl" at startup
    POSName:  String;              // Script to run at startup
    procedure LoadConfig;          // Load configuration details
  end;

Var ConfigForm: TConfigForm;

Implementation

Uses BFOOCore;     // IBM 1401 core storage system

{$R *.DFM}

Procedure SetCBValue(CB: TCheckBox; VA: Boolean; Var VV: Boolean);
Begin
  VV:=VA;           // Set variable for access by other units
  CB.Checked:=VA;   // And set checkbox
End;

Procedure SetEBValue(EB: TEdit; VA: String; Var VV: String);
Begin
  VV:=VA;           // Set variable for access by other units
  EB.Text:=VA;      // And set editbox
End;

procedure TConfigForm.FormCreate(Sender: TObject);
Var SS: String;
begin
  SS:=ExePath+ExeName+'.INI';
  IniFile:=TIniFile.Create(SS);
end;

procedure TConfigForm.FormDestroy(Sender: TObject);
begin
  IniFile.Free;
end;

procedure TConfigForm.LoadConfig;
begin
  With IniFile do begin
//  SetCBValue(CBPowerOn,ReadBool('Operation','PowerOn',True),PowerOn);
    SetEBValue(EBPOSName,ReadString('Operation','StartupScript','PowerOn.OCL'),POSName);
    SetCBValue(CBCoreInitCheck,ReadBool('Operation','CoreInitCheck',False),CoreInitCheck);
    Case ReadInteger('Core Storage','Size',1400) of
       1400: RBCS1.Checked:=True;
       2000: RBCS2.Checked:=True;
       4000: RBCS3.Checked:=True;
       8000: RBCS4.Checked:=True;
      12000: RBCS5.Checked:=True;
      16000: RBCS6.Checked:=True;
    End;
  End;
end;

procedure TConfigForm.RBCSClick(Sender: TObject);
begin
  With (Sender As TComponent) do
    CoreSize:=Tag;
end;

procedure TConfigForm.PBOKClick(Sender: TObject);
begin
  EBPOSName.Text:=Trim(EBPOSName.Text);
  If EBPOSName.Text<>'' then
    EBPOSName.Text:=ChangeFileExt(EBPOSName.Text,'.OCL');
  With IniFile do begin
//  WriteBool('Operation','PowerOn',CBPowerOn.Checked);
    DeleteKey('Operation','PowerOn');
    WriteString('Operation','StartupScript',EBPOSName.Text);
    WriteBool('Operation','CoreInitCheck',CBCoreInitCheck.Checked);
    WriteInteger('Core Storage','Size',CoreSize);
    UpdateFile;
  End;
  CoreForm.SetCoreSize(CoreSize);
end;

procedure TConfigForm.PBBrowseClick(Sender: TObject);
begin
  With OpenDialog do
    If Execute then
      EBPosName.Text:=FileName;
end;

procedure TConfigForm.FormShow(Sender: TObject);
begin
  LoadConfig;
end;

end.
