//******************************************************************************
// This unit contains global, utility, and miscellaneous definitions for the
// IBM 1401 emulator
//******************************************************************************
Unit BFOODefs;

Interface

Uses SysUtils,Windows,Messages,Forms,
     IBMDefs;      // Global, utility, and miscellaneous definitions

//*********************************************************
// Internally, actual 1401 BCD character codes are used, not ASCII
// These are the BCD codes for all characters used
//*********************************************************
Const Ch_BLANK=$00;
      Ch_1    =$01;
      Ch_2    =$02;
      Ch_3    =$03;
      Ch_4    =$04;
      Ch_5    =$05;
      Ch_6    =$06;
      Ch_7    =$07;
      Ch_8    =$08;
      Ch_9    =$09;
      Ch_0    =$0A;
      Ch_HASH =$0B;     // #
      Ch_ATSGN=$0C;     // @

      Ch_TMARK=$0F;     // Reads from tape as $8F - (high bit=IRG)

      Ch_TBLNK=$10;     // " "
      Ch_SLASH=$11;     // "/"
      Ch_S    =$12;
      Ch_T    =$13;
      Ch_U    =$14;
      Ch_V    =$15;
      Ch_W    =$16;
      Ch_X    =$17;
      Ch_Y    =$18;
      Ch_Z    =$19;
      Ch_RMARK=$1A;     // Record Mark
      Ch_COMMA=$1B;     // ,
      Ch_PERCT=$1C;     // %
      Ch_WDSEP=$1D;     // Word Separator

      Ch_DASH =$20;     // -
      Ch_J    =$21;
      Ch_K    =$22;
      Ch_L    =$23;
      Ch_M    =$24;
      Ch_N    =$25;
      Ch_O    =$26;
      Ch_P    =$27;
      Ch_Q    =$28;
      Ch_R    =$29;
      Ch_EXCLM=$2A;     // !
      Ch_DOLAR=$2B;     // $
      Ch_ASTER=$2C;     // *

      Ch_AMPER=$30;     // &
      Ch_A    =$31;
      Ch_B    =$32;
      Ch_C    =$33;
      Ch_D    =$34;
      Ch_E    =$35;
      Ch_F    =$36;
      Ch_G    =$37;
      Ch_H    =$38;
      Ch_I    =$39;
      Ch_QUEST=$3A;     // ?
      Ch_FSTOP=$3B;     // .
      Ch_LOZEN=$3C;     // Lozenge

      Ch_GMARK=$3F;     // Group Mark

Const MaxCore=16000;              // Maximum possible memory size

//*********************************************************
// Emulated 1401 CPU related variables
//*********************************************************
// Current instruction values
Var InstAddr: Word;               // Address of current instruction
    InstLen:  Byte;               // Length of current instruction

// CPU Registers
Var IAReg:    Word;               // Instruction Address register
    AAReg:    Word;               // A-Address register
    BAReg:    Word;               // B-Address register
    OReg:     Byte;               // Opcode register
    DReg:     Byte;               // D-Char register

// CPU Indicators
Var ECInd:    Boolean;            // Equal Compare
//  UCInd:    Boolean;            // Unequal Compare
    LCInd:    Boolean;            // Low Compare
//  HCInd:    Boolean;            // High Compare
    OVInd:    Boolean;            // Overflow

// Console Sense switches A..G
Var Switches: Array[Ch_A..Ch_G] Of Boolean;

//  I/O interface
Var XFerCount:  Word;             // Chars transfered by last I/O operation
    TapeIndicate: Boolean;
    TapeError:    Boolean;

// Debug stops
Var StopAddr:   Word;             // Address to stop at
    CMRDAddr:   Word;             // Address for stop on Core Read
    CMWRAddr:   Word;             // Address for stop on Core Write
    StopWVal:   Byte;             // Core write value to stop with
    StopOpcd:   Byte;             // OpCode to stop at
    StopMesg:   String;           // Explanation of stop

//*********************************************************
// Emulator related definitions
//*********************************************************
// Window identifiers
Type TFormID=(FINONE,FICore,FIReader,FIPrinter,FITape,FITrace);

// Window handles
Var MainFormHandle:     HWnd;
    DisplayFormHandle:  HWnd;
    ReaderFormHandle:   HWnd;
    PrinterFormHandle:  HWnd;
    TapeUnitFormHandle: HWnd;

// Internally generated Message codes
Const WM_SHOWSTATUS    =WM_APP+301;
      WM_CONSOLECOMMAND=WM_APP+302;
      WM_DISPSHOW      =WM_APP+303;    // OCLDisplayForm Show/Hide
      WM_IOCONTROL     =WM_APP+304;
      WM_CPUSTOPPED    =WM_APP+305;
      WM_ZAP           =WM_APP+306;

// Parameter values for WM_CONSOLECOMMAND messages
Const CCSETBUTTONS   =01;         // Update console button states
      CCRESET        =05;         // Reset emulator
      CCSTART        =15;         // Push Start button
      CCSHOWFORM     =20;         // Show a window
      CCHIDEFORM     =21;         // Hide a window
      CCSWITCHON     =22;         // Set a sense switch on
      CCSWITCHOFF    =23;         // Set a sense switch off
      CCTURBOON      =24;         // Set Turbo mode On
      CCTURBOOFF     =25;         // Set Turbo mode Off
      CCSETCORSIZ    =30;         // Set the core storage size

// Record structure used by WM_IOCONTROL messages
Type PString=^String;             // Type definition for a string pointer
     PIOControl=^TIOControl;
     TIOControl=Packed Record
       Action:    Word;           // I/O operation action code (see below)
       SubParm:   Word;           // Additional operation parameter
       Options:   TIOOptions;     // Processing options
       StrParm:   PString;        // Pointer to any required string parameter
     End;

// Action codes used by WM_IOCONTROL messages
Const FCRESET  =1;                // Reset I/O device
      FCCLEAR  =2;                // Clear I/O device
      FCOPEN   =3;                // Open a Windows file for I/O
      FCLOAD   =4;                // Load record to core storage for IPL
      FCSAVE   =5;                // Save I/O data to Windows file
      FCREAD   =6;                // Read a record
      FCWRITE  =7;                // Write a record
      FCWRBLNK =8;                // Write a blank record
      FCCOUNT  =9;                // Update IOCount variable with device's count
      FCOPTIONS=10;               // Set options for next I/O operation
      // 1402 specific
      FCRESTAK=20;                // Move last card read/punched to another stacker
      FCDUMMY =21;                // Set up a dummy card that will always be read
      // Tape specific
      FCRSTIND=30;                // Reset Tape Indicate state
      FCPROTON=31;                // Set Write protect status
      FCPROTOF=32;                // Reset Write protect status
      FCREWIND=33;                // Rewind tape
      FCREWUNL=34;                // Rewind and Unload tape
      FCWTMARK=35;                // Write tape mark

// Record structure used by WM_ZAP messages
Type PZapMsg=^TZapMsg;
     TZapMsg=Packed Record
       Target:    Word;           // Patch target
       Address:   LongWord;       // Address to modify
       Value:     Word;           // New value
//       Unused2:   Word;
     End;

Var  IOCount:        Word;        // I/O count set by FCCOUNT action
     TurboMode:      Boolean;     // Skip display updates to run faster
     BaseScriptPath: String;      // Name of main script file

// Exception Set
Type E1401Exception=Class(Exception)        // Base class
       Address: Word;
     End;
     EProcessCheck=Class(E1401Exception);   // CPU check
     EIOCheck=Class(E1401Exception);        // I/O check
     ECoreWatch=Class(E1401Exception);      // Core stop or exception

Procedure SendIOCMsg(WH: HWnd; AC: Word; SU: Word; IO: TIOOptions; SS: String);
Procedure SendZapMsg(WH: HWnd; SU: Word; AD: LongWord; NV: Byte);
Procedure RaiseProcessCheck(CM: String; AD: Word);
Procedure RaiseIOCheck(CM: String; AD: Word);

Implementation

Procedure RaiseProcessCheck(CM: String; AD: Word);
Var EX: EProcessCheck;
Begin
  EX:=EProcessCheck.Create(CM);
  With EX do begin
    Address:=AD;
//  Message:=Message+' '+IntToStr(AD);
  End;
  Raise EX;
End;

Procedure RaiseIOCheck(CM: String; AD: Word);
Var EX: EIOCheck;
Begin
  EX:=EIOCheck.Create(CM);
  With EX do begin
    Message:=CM;
    Address:=AD;
  End;
  Raise EX;
End;

Procedure SendIOCMsg(WH: HWnd; AC: Word; SU: Word; IO: TIOOptions; SS: String);
Var MR: TIOControl;
Begin
  With MR do begin
    Action:=AC;
    SubParm:=SU;
    Options:=IO;
    If SS='' then StrParm:=NIL
             else StrParm:=@SS;
  End;
  SendMessage(WH,WM_IOCONTROL,0,Longint(@MR));
End;

Procedure SendZapMsg(WH: HWnd; SU: Word; AD: LongWord; NV: Byte);
Var PM: TZapMsg;
Begin
  With PM do begin
    Target:=SU;
    Address:=AD;
    Value:=NV;
  End;
  SendMessage(WH,WM_ZAP,0,Longint(@PM));
End;

End.
