//******************************************************************************
// This unit provides display and control functions for the Operator Control
// Language (OCL) scripting langauge interpreter used by the IBM 1401 emulator.
//******************************************************************************
Unit BFOODOCL;

Interface

Uses Windows, Messages, SysUtils, Classes,
     Graphics, Controls, Forms, Dialogs, StdCtrls,
     BFOODefs;     // Global, utility, and miscellaneous definitions

// Types of display items that can be generated on the form
Type TDispItemTypes=(
       DIBlankLine,    // Blank space
       DITextLabel,    // A simple text label
       DIClikLabel     // A label that can be clicked
     );

Type
  TOCLDisplayForm = class(TForm)
    PBContinue: TButton;
    PBCancel: TButton;
    procedure FormCreate(Sender: TObject);
    procedure PBContinueClick(Sender: TObject);
    procedure PBCancelClick(Sender: TObject);
    procedure ClickHandler(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    ActiveCtr:  Integer;      // Number of active (clickable) items
    XPos,YPos:  Integer;      // Position to place next item on form
    MaxWid:     Integer;      // Width of longest item
    LastHeight: Integer;      // Height of last item created
  protected
    procedure WMDispShow(Var Message: TMessage); Message WM_DISPSHOW;
  public
    SelectIndex: Integer;     // Indicates which active item was selected
    ContEnable:  Boolean;     // Show "Continue" button
    TColour:   TColor;
    HColour:   TColor;
    Procedure ClearDispItems;
    Procedure AddDispItem(DS: String; DT: TDispItemTypes);
  end;

Var OCLDisplayForm: TOCLDisplayForm;

Implementation

{$R *.DFM}

Uses BFOOIOCL;     // Operator Control Language (OCL) interpreter

procedure TOCLDisplayForm.FormCreate(Sender: TObject);
begin
  Inherited;
  DisplayFormHandle:=Handle;
  TColour:=CLBlack;
end;

procedure TOCLDisplayForm.FormDestroy(Sender: TObject);
begin
  OCLDisplayForm:=NIL;  // To stop abend in scripter reset
end;

procedure TOCLDisplayForm.ClearDispItems;
Var XC: Integer;
    CO: TControl;
begin
  For XC:=ControlCount-1 Downto 0 do begin
    CO:=Controls[XC];
    If CO.Tag<0 then Continue;   // Don't delete the OK & Cancel buttons
    RemoveControl(CO);
    CO.Free;
  End;
  ActiveCtr:=0; MaxWid:=200;
  XPos:=10; YPos:=10;
end;

procedure TOCLDisplayForm.AddDispItem(DS: String; DT: TDispItemTypes);
Var DI: TControl;
begin
  Case DT of
    DIBlankLine:  DI:=NIL;
    DITextLabel:  Begin
                    DI:=TLabel.Create(NIL);
                    With TLabel(DI) do begin
                      Caption:=DS;
                      With Font do begin
                        Size:=9; Color:=TColour; Style:=[];
                      End;
                    End;
                    DI.Tag:=0;
                  End;
    DIClikLabel:  Begin
                    DI:=TLabel.Create(NIL);
                    With TLabel(DI) do begin
                      Caption:=DS;
                      With Font do begin
                        Size:=9; Color:=HColour; Style:=[FSBold];
                      End;
                      OnClick:=ClickHandler;
                    End;
                    DI.Cursor:=CRHandPoint;
                    Inc(ActiveCtr);
                    DI.Tag:=ActiveCtr;
                  End;
    Else          Exit;
  End;
  If DI<>NIL then begin
    With DI do begin
      Parent:=Self; Visible:=True;
      Left:=XPos; Top:=YPos;
      If Width>MaxWid then MaxWid:=Width;
      LastHeight:=Height;
    End;
  End else Begin
    If LastHeight<>0 then Inc(YPos,LastHeight)
                     else Inc(YPos,15);
    LastHeight:=0;
  End;
  Width:=10+MaxWid+15;
  Height:=YPos+10+70;
End;

procedure TOCLDisplayForm.ClickHandler(Sender: TObject);
begin
  Inherited;
  Hide;
  With (Sender As TControl) do
    SelectIndex:=Tag;
//OCLThread.ResumeScript;
  OCLThread.Resume;
end;

procedure TOCLDisplayForm.PBContinueClick(Sender: TObject);
begin
  Inherited;
  Hide;
//OCLThread.ResumeScript;
  OCLThread.Resume;
end;

procedure TOCLDisplayForm.PBCancelClick(Sender: TObject);
begin
  Inherited;
  Hide;
  OCLThread.TerminateScript;
end;

procedure TOCLDisplayForm.WMDispShow(Var Message: TMessage);
begin
  SelectIndex:=0;
  PBContinue.Visible:=ContEnable;
  If Message.WParam=0 then Show
                      else Hide;
end;

End.

