//******************************************************************************
// This is the main form unit of the IBM 1401 emulator.
// It provides the controls and indicators of the emulated 1401 CPU, and the
// controls required for overall emulator operation.
//******************************************************************************
Unit BFOOMain;

Interface

Uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, ExtCtrls, StdCtrls, Menus,
     IBMDefs,      // Global, utility, and miscellaneous definitions
     IBMChars,     // Character codes and mappings
     IBMTrace,     // Log/Trace display
     IBM1402,      // IBM 1402 card reader/punch
     IBM1403,      // IBM 1403 line printer
     IBM729IV,     // IBM 729 model IV magnetic tape unit
     BFOOAbot,     // "About" box
     BFOODefs,     // Global, utility, and miscellaneous definitions
     BFOOCnfg,     // Configuration and Options
     BFOOCore,     // IBM 1401 core storage system
     BFOOProc,     // IBM 1401 Central Processing Unit
     BFOOEOCL,     // Simple OCL script editor
     BFOOIOCL;     // Operator Control Language (OCL) interpreter

Type
  TMainForm=Class(TForm)
    PAMain: TPanel;
    PBStart: TButton;
    GroupBox1: TGroupBox;
    RBRUN: TRadioButton;
    RBSTEP: TRadioButton;
    RBAdSt: TLabel;
    PBReset: TButton;
    EBAdSt: TEdit;
    RBCRSt: TLabel;
    EBCRSt: TEdit;
    PBStop: TButton;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    LAIAR: TLabel;
    LAAAR: TLabel;
    Label11: TLabel;
    LABAR: TLabel;
    Label13: TLabel;
    Label6: TLabel;
    LAOP: TLabel;
    RBCWSt: TLabel;
    EBCWSt: TEdit;
    RBOCSt: TLabel;
    EBOCSt: TEdit;
    PBExit: TButton;
    LAStatus: TLabel;
    Label2: TLabel;
    LAILen: TLabel;
    GroupBox3: TGroupBox;
    LAWTrace: TLabel;
    LAWCore: TLabel;
    LAWRdrPun: TLabel;
    LAWPRT: TLabel;
    LAWCHR: TLabel;
    MainMenu: TMainMenu;
    MIFile: TMenuItem;
    MIOpen: TMenuItem;
    N1: TMenuItem;
    MIExit: TMenuItem;
    MIView: TMenuItem;
    MIAbout: TMenuItem;
    MIOptions: TMenuItem;
    MICore: TMenuItem;
    MIRDR: TMenuItem;
    MIPRT: TMenuItem;
    N2: TMenuItem;
    MIChars: TMenuItem;
    N3: TMenuItem;
    MITrace: TMenuItem;
    MIHelp: TMenuItem;
    OpenDialog: TOpenDialog;
    GroupBox4: TGroupBox;
    CBSRA: TCheckBox;
    CBSRB: TCheckBox;
    CBSRE: TCheckBox;
    CBSRD: TCheckBox;
    CBSRG: TCheckBox;
    CBSRF: TCheckBox;
    CBSRC: TCheckBox;
    PBResume: TButton;
    LAWTape: TLabel;
    PBScript: TButton;
    MIEdit: TMenuItem;
    CBTurbo: TCheckBox;
    EBWVSt: TEdit;
    procedure PBExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PBStartClick(Sender: TObject);
    procedure StopChange(Sender: TObject);
    procedure PBResetClick(Sender: TObject);
    procedure PBStopClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure LAWindowClick(Sender: TObject);
    procedure MIOpenClick(Sender: TObject);
    procedure CBSRClick(Sender: TObject);
    procedure MIAboutClick(Sender: TObject);
    procedure PBScriptClick(Sender: TObject);
    procedure MIEditClick(Sender: TObject);
    procedure MIOptionsClick(Sender: TObject);
    procedure CBTurboClick(Sender: TObject);
  private
    Initialized: Boolean;
    procedure RunScript(SP: String);
  protected
    procedure WMShowStatus(Var Message: TMessage); Message WM_SHOWSTATUS;
    procedure WMCPUStopped(Var Message: TMessage); Message WM_CPUSTOPPED;
    procedure WMConsoleCommand(Var Message: TMessage); Message WM_CONSOLECOMMAND;
  public
    procedure ResetCPU;
    procedure ResetWindows;
    procedure SetButtons;
  end;

Var MainForm: TMainForm;

Implementation

{$R *.DFM}

Const Heading='IBM 1401 Emulator';

//*********************************************************
// Form events
//*********************************************************
procedure TMainForm.FormCreate(Sender: TObject);
begin
  MainFormHandle:=Self.Handle;
  Top:=0; Left:=0;
  Caption:=Heading;
  Application.ShowHint:=True;
end;

procedure TMainForm.FormActivate(Sender: TObject);
Var SS: String;
begin
  If Not Initialized then begin
    Initialized:=True;
    // Position windows
    With CoreForm do begin
      Top:=Self.Height; Left:=0;
      SetCoreSize(8000);
    End;
    With ReaderForm do begin
      Top:=Screen.Height-Height-30; Left:=0;
    End;
    With PrinterForm do begin
      Top:=0; Left:=Self.Width;
    End;
    With TraceForm do begin
      Top:=Screen.Height-Height-30; Left:=Screen.Width-Width;
    End;
    UseCMCMapping:=True;
    // Set control defaults
    RBRUN.Checked:=True;
    ResetWindows;
    ResetCPU;
    // Load Configuration
    ConfigForm.LoadConfig;
    // Auto run from command line or name in config file
    If ParamCount>0 then begin
      RunScript(ParamStr(1));
      Exit;
    End;
    SS:=ConfigForm.POSName;
    If FileExists(SS) then begin
      RunScript(SS);
      Exit;
    End;
  End;
end;

//*********************************************************
// Control events
//*********************************************************
procedure TMainForm.PBExitClick(Sender: TObject);
begin
  Close;
end;

// Configuration display
procedure TMainForm.MIOptionsClick(Sender: TObject);
begin
  ConfigForm.ShowModal;
end;

// About button
procedure TMainForm.MIAboutClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

// Script open menu item
procedure TMainForm.MIOpenClick(Sender: TObject);
begin
  With OpenDialog do begin
    InitialDir:=EXEPath;
    If Execute then
      RunScript(FileName);
  End;
  SetButtons;
End;

// Script edit menu item
procedure TMainForm.MIEditClick(Sender: TObject);
begin
  If BaseScriptPath<>'' then
    OCLEditForm.Show;
end;

// Reset button
procedure TMainForm.PBResetClick(Sender: TObject);
begin
  // These resets are done only by the RESET button
  OCLThread.Reset;
  TraceForm.Clear;
{ FillChar(Switches,SizeOf(Switches),#0);
  CBSRA.Checked:=False; CBSRB.Checked:=False; CBSRC.Checked:=False;
  CBSRD.Checked:=False; CBSRE.Checked:=False; CBSRF.Checked:=False;
  CBSRG.Checked:=False;}
  // Now do other resets (Same as script RESET command)
  PostMessage(MainFormHandle,WM_CONSOLECOMMAND,CCRESET,0);
end;

// Script button
procedure TMainForm.PBScriptClick(Sender: TObject);
begin
  RunScript(BaseScriptPath);
end;

// Start button
procedure TMainForm.PBStartClick(Sender: TObject);
Begin
  With CPUThread do begin
{   If TComponent(Sender).Tag=0 then begin
      If RBRun.Checked then RunState:=RSRUNN       // Start button
                       else RunState:=RSSTEP;
    End else Begin
      RBRun.Checked:=True;
      RunState:=RSRUNN;                            // Resume button
    End;
    Resume;        // Restart CPU instruction execution}
    If TComponent(Sender).Tag=1 then begin  // Resume button
      RBRun.Checked:=True;
      CPUThread.Resuming:=True;
    End;
    If RBRun.Checked then RunState:=RSRUNN  // Set Run/Step mode
                     else RunState:=RSSTEP;
    Resume;                                 // Restart CPU instruction execution
  End;
  SetButtons;
End;

// Operator stop
procedure TMainForm.PBStopClick(Sender: TObject);
begin
  StopMesg:='Operator Halt at';
  CPUThread.RunState:=RSOPER;
  SetButtons;
end;

// Run/Step mode changed
procedure TMainForm.StopChange(Sender: TObject);
Var SO: String;
begin
  // Load stop conditions from controls
  StopAddr:=StrToInt(EBAdSt.Text);
  CMRDAddr:=StrToInt(EBCRSt.Text);
  CMWRAddr:=StrToInt(EBCWSt.Text);
  SO:=EBWVSt.Text+' '; StopWVal:=ASC2BCD[Ord(SO[1])];
  SO:=EBOCSt.Text+' '; StopOpCd:=ASC2BCD[Ord(SO[1])];
  If RBStep.Checked then TraceForm.InstTrace:=True;
end;

// Sense switch clicked
procedure TMainForm.CBSRClick(Sender: TObject);
begin
  With TCheckBox(Sender) do
    Switches[Tag+$30]:=Checked;
end;

// Turbo button clicked
procedure TMainForm.CBTurboClick(Sender: TObject);
begin
  TurboMode:=CBTurbo.Checked;
end;

// Show/Hide emulator windows
procedure TMainForm.LAWindowClick(Sender: TObject);
begin
  With TLabel(Sender) do
    Case Tag of
      1: With TraceForm    do Visible:=Not Visible;
      2: With CoreForm     do Visible:=Not Visible;
      3: With ReaderForm   do Visible:=Not Visible;
      4: With PrinterForm  do Visible:=Not Visible;
      5: With TapeUnitForm do Visible:=Not Visible;
      6: With CharMapForm  do Visible:=Not Visible;
    End;
end;

// Message handler to update console registers display
Procedure TMainForm.WMShowStatus(Var Message: TMessage);
begin
  LAILen.Caption:=IntToStr(InstLen);
  LAIAR.Caption:=W2S(IAReg);
  LAAAR.Caption:=W2S(AAReg);
  LABAR.Caption:=W2S(BAReg);
  LAOP.Caption:=ASC(OReg);
  Inherited;
End;

// Message handler to perform console operations
Procedure TMainForm.WMConsoleCommand(var Message: TMessage);
Var WB: Boolean;
Begin
  With Message do
    Case WParam of
      CCSETBUTTONS:    SetButtons;
      CCRESET:         Begin
                         ResetWindows;
                         ResetCPU;
                         SetButtons;
                       End;
      CCSTART:         PBStart.Click;
      CCSETCORSIZ:     CoreForm.SetCoreSize(LParam);
      CCSHOWFORM,
      CCHIDEFORM:      Begin
                         WB:=WParam=CCSHOWFORM;
                         Case TFormID(LParam) Of
                           FICORE:    CoreForm.Visible:=WB;
                           FIREADER:  ReaderForm.Visible:=WB;
                           FIPRINTER: PrinterForm.Visible:=WB;
                           FITAPE:    TapeUnitForm.Visible:=WB;
                           FITRACE:   TraceForm.Visible:=WB;
                         End;
                       End;
      CCSWITCHON,
      CCSWITCHOFF:     Begin
                         WB:=WParam=CCSWITCHON;
                         Case LParam Of
                           1: CBSRA.Checked:=WB;
                           2: CBSRB.Checked:=WB;
                           3: CBSRC.Checked:=WB;
                           4: CBSRD.Checked:=WB;
                           5: CBSRE.Checked:=WB;
                           6: CBSRF.Checked:=WB;
                           7: CBSRG.Checked:=WB;
                         End;
                       End;
      CCTURBOON,
      CCTURBOOFF:      CBTurbo.Checked:=WParam=CCTURBOON;
    End;
  Application.ProcessMessages;
End;

// Message handler to process CPU stop condition
procedure TMainForm.WMCPUStopped(Var Message: TMessage);
Begin
  SetButtons;
  If (CPUThread.RunState<>RSOPER) And (OCLThread.ScriptState=SSWAIT) then
    OCLThread.Resume;
End;

// Reset all display windows
procedure TMainForm.ResetWindows;
begin
  CoreForm.Clear;
  SendIOCMsg(ReaderFormHandle,FCRESET,0,[],'');
  SendIOCMsg(PrinterFormHandle,FCRESET,0,[],'');
  SendIOCMsg(TapeUnitFormHandle,FCRESET,0,[],'')
End;

// Reset the CPU
procedure TMainForm.ResetCPU;
begin
  CPUThread.reset;
  RBRun.Checked:=True;
  SetButtons;
end;

// Set button enabled status according to current emulator state
procedure TMainForm.SetButtons;
Begin
  PBScript.Enabled:=(BaseScriptPath<>'') And
                    (OCLThread.ScriptState<>SSRUNN) And
                    (CPUThread.RunState<>RSRUNN);
  MIEdit.Enabled:=PBScript.Enabled;
  LAStatus.ParentColor:=True;
  LAStatus.Font.Color:=CLWhite;
//Should this replace separate assignments below??
//LAStatus.Caption:=TCPUThread.RunStateString;
  With CPUThread do begin
    Case RunState of
      RSINIT: Begin
                LAStatus.Caption:='System Reset';
                PBStart.Enabled:=False;
                PBStop .Enabled:=False;
                PBReset.Enabled:=True;
                PBExit .Enabled:=True;
              End;
      RSCLOD: Begin
                LAStatus.Caption:='Card Loaded';
                PBStart.Enabled:=True;
                PBStart.SetFocus;
              End;
      RSTLOD: Begin
                LAStatus.Caption:='Tape record Loaded';
                PBStart.Enabled:=True;
                PBStart.SetFocus;
              End;
      RSOPER,RSSTOP,RSSTEP,
      RSHALT: Begin
                LAStatus.Caption:=StopMesg+' '+W2S(InstAddr);
                If RunState in [RSSTOP,RSSTEP] then begin
                  RBStep.Checked:=True;
                  PBResume.Enabled:=True;
                End;
                PBStart.Enabled:=True;
                PBStop .Enabled:=False;
                PBReset.Enabled:=True;
                PBExit .Enabled:=True;
                PBStart.SetFocus;
              End;
      RSRUNN: Begin
                LAStatus.Caption:='Running';
                PBStart.Enabled:=False;
                PBStop .Enabled:=True;
                PBReset.Enabled:=False;
                PBExit .Enabled:=False;
                PBStop.SetFocus;
              End;
      RSCHEK: Begin
                LAStatus.Color:=CLRed;
                LAStatus.Caption:={CPUThread.RunStateString+}StopMesg;
                PBStop .Enabled:=False;
                PBReset.Enabled:=True;
                PBExit .Enabled:=True;
                PBReset.SetFocus;
              End;
    End;
    EBAdSt.Enabled:=RunState<>RSRUNN;
    EBCRSt.Enabled:=RunState<>RSRUNN;
    EBCWSt.Enabled:=RunState<>RSRUNN;
    EBWVSt.Enabled:=RunState<>RSRUNN;
    EBOCSt.Enabled:=RunState<>RSRUNN;
  End;
  // Update resgister display
  PostMessage(MainFormHandle,WM_ShowStatus,0,0);
  Application.ProcessMessages;
End;

// Run an OCL script file
Procedure TMainForm.RunScript(SP: String);
Var PL: TPList;
Begin
  FillChar(PL,SizeOf(PL),#0);                    // Clear PList and
  PL[0]:=SP;                                     // Insert exec name
  BaseScriptPath:=SP;                            // Note for re-run/editing
  Caption:=Heading+' - '+ExtractFileName(SP);    // Show in caption
  OCLThread.Reset;                               // Drop any existing scripts
  OCLThread.RunScript(SP,PL);                    // Load and run new script
  SetButtons;
End;

End.

