//******************************************************************************
// This unit implements the functions of the IBM 1402 card reader/punch.
// Other units send Windows messages to this form to invoke the required I/O
// operations. The unit maintains multiple lists of card images, one for each
// stacker of the 1402. Radio buttons control which stacker is is displayed.
// The unit employs a specialized listbox component "TUnitRecordDisplay" to
// enable the cards to be displayed in reverse order, with the first card at
// the bottom of the list. Other buttons allow a file to selected for loading,
// cause the current deck to be reloaded to the input hopper, and cause the
// first card to be read into core storage.
//******************************************************************************
Unit IBM1402;

Interface

Uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, ExtCtrls,StdCtrls, ComCtrls,
     URecDisp,     // Unit record display component
     IBMDefs,      // Global, utility, and miscellaneous definitions
     IBMChars,     // Character codes and mappings
     BFOODefs,     // Global, utility, and miscellaneous definitions
     BFOOCore;     // IBM 1401 core storage system

Type TStacker=(SNONE,SRDRN,SRDR1,SRP82,SPUN4,SPUNN);

Type
  TReaderForm = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    LACardCount: TLabel;
    Hopper: TUnitRecordDisplay;
    CBSize: TCheckBox;
    RBRdr: TRadioButton;
    RBStkrNR: TRadioButton;
    RBStkrR1: TRadioButton;
    RBStkr82: TRadioButton;
    RBStkrP4: TRadioButton;
    RBStkrNP: TRadioButton;
    PBLoad: TButton;
    PBOpen: TButton;
    OpenDialog: TOpenDialog;
    PBDeck: TButton;
    procedure FormResize(Sender: TObject);
    procedure CBSizeClick(Sender: TObject);
    procedure RBClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PBLoadClick(Sender: TObject);
    procedure PBOpenClick(Sender: TObject);
    procedure PBDeckClick(Sender: TObject);
  private
    ReadDeck: TStringList;
    RDRNDeck: TStringList;
    RDR1Deck: TStringList;
    RP82Deck: TStringList;
    PUN4Deck: TStringList;
    PUNNDeck: TStringList;
    LastWasRead: Boolean;
    LastDir:     String;
    DeckName: String;
    Check:    Boolean;
    DummyCard: TCard;
    procedure Refresh;
    procedure ReadFirstCard;
    procedure ReadCard;
    procedure PunchCard;
    procedure ReStackCard(SN: Byte);
  protected
    procedure WMSetSize(Var Message: TMessage); Message WM_SetSize;
    procedure WMIOControl(Var Message: TMessage); Message WM_IOCONTROL;
  public
    Function  CardsLeft: Word;
  End;

Var ReaderForm: TReaderForm;

Implementation

{$R *.DFM}

Uses BFOOProc,     // IBM 1401 Central Processing Unit
     BFOOMain;     // Main emulator window

procedure TReaderForm.FormCreate(Sender: TObject);
Var PS: String;
begin
  Inherited;
  Caption:='IBM Emulation - 1402 Card Reader/Punch';
  ReadDeck:=TStringList.Create;
  RDRNDeck:=TStringList.Create;
  RDR1Deck:=TStringList.Create;
  RP82Deck:=TStringList.Create;
  PUN4Deck:=TStringList.Create;
  PUNNDeck:=TStringList.Create;
  Hopper.Records:=ReadDeck;
  ReaderFormHandle:=Handle;
  LastDir:=EXEPath;
  // Autoload a deck from command line parameter
  If ParamCount>0 then begin
    PS:=ParamStr(1);
    If (Pos('.CRD',PS)>0) Or
       (Pos('.CDR',PS)>0) then begin
      DeckName:=PS;
      PBDeckClick(Self);
    End;
  End;
end;

procedure TReaderForm.FormDestroy(Sender: TObject);
begin
  ReadDeck.Free;
  RDRNDeck.Free;
  RDR1Deck.Free;
  RP82Deck.Free;
  PUN4Deck.Free;
  PUNNDeck.Free;
end;

Procedure TReaderForm.FormResize(Sender: TObject);
begin
  Hopper.SetSize; // Sends a WM_SetSize when done
end;

Procedure TReaderForm.WMSetSize(Var Message: TMessage);
begin
  Inherited;
  Width:=Hopper.Width+8;
End;

procedure TReaderForm.CBSizeClick(Sender: TObject);
begin
  If Not CBSize.Checked then Hopper.Font.Size:=6
                        else Hopper.Font.Size:=8;
end;

procedure TReaderForm.RBClick(Sender: TObject);
Var RD: TStringList;
begin
  Case TComponent(Sender).Tag of
    0:   RD:=ReadDeck;
    1:   RD:=RDRNDeck;
    2:   RD:=PUNNDeck;
    3:   RD:=RDR1Deck;
    4:   RD:=PUN4Deck;
    5:   RD:=RP82Deck;
    Else RD:=NIL;
  End;
  Hopper.Records:=RD;
  Refresh;
End;

procedure TReaderForm.Refresh;
begin
  Hopper.Refresh;
  LACardCount.Caption:=IntToStr(Hopper.Records.Count);
  PBDeck.Enabled:=DeckName<>'';
  PBLoad.Enabled:=Hopper.Records.Count>0;
End;

function TReaderForm.CardsLeft: Word;
begin
  Result:=ReadDeck.Count;
end;

// Message handler for I/O commands
Procedure TReaderForm.WMIOControl(Var Message: TMessage);
begin
  With PIOControl(Message.LParam)^ do begin
    If Check And (Action<>FCRESET) then Exit;
    Case Action Of
      FCRESET:  Begin
                  ReadDeck.Clear;
                  RDRNDeck.Clear;
                  RDR1Deck.Clear;
                  RP82Deck.Clear;
                  PUN4Deck.Clear;
                  PUNNDeck.Clear;
                  RBRdr.Checked:=True;
                  Check:=False;
                End;
      FCCLEAR:  Case TStacker(SubParm) of
                  SRDRN: RDRNDeck.Clear;
                  SRDR1: RDR1Deck.Clear;
                  SPUNN: PUNNDeck.Clear;
                  SPUN4: PUN4Deck.Clear;
                  SRP82: RP82Deck.Clear;
                End;
      FCSAVE:   Case TStacker(SubParm) of
                  SRDRN: RDRNDeck.SaveToFile(StrParm^);
                  SRDR1: RDR1Deck.SaveToFile(StrParm^);
                  SPUNN: PUNNDeck.SaveToFile(StrParm^);
                  SPUN4: PUN4Deck.SaveToFile(StrParm^);
                  SRP82: RP82Deck.SaveToFile(StrParm^);
                End;
      FCOPEN:   CharMapForm.ProcessCardFile(StrParm^,ReadDeck,Options);
      FCLOAD:   ReadFirstCard;
      FCREAD:   ReadCard;
      FCWRITE:  PunchCard;
      FCRESTAK: ReStackCard(SubParm);
      FCDUMMY:  Begin
                  DummyCard:=UpperCase(StrParm^);
                  While Length(DummyCard)<80 do DummyCard:=DummyCard+' ';
                End;
      Else      RaiseIOCheck('Invalid I/O Op for RDR/PUN: '+IntToStr(Action),0);
    End;
  End;
  If Not TurboMode then
    Refresh;
End;

procedure TReaderForm.PBOpenClick(Sender: TObject);
begin
  With OpenDialog do begin
    DefaultExt:='CRD';
    Filter:='Card files (*.CRD *.CDR)|*.CRD;*.CDR|All files (*.*)|*.*';
    InitialDir:=LastDir;
    If Execute then begin
      DeckName:=FileName;
      LastDir:=ExtractFilePath(FileName);
      PBDeckClick(Self);
    End;
  End;
  Refresh;
End;

procedure TReaderForm.PBDeckClick(Sender: TObject);
begin
  Try
    CharMapForm.ProcessCardFile(DeckName,ReadDeck,[]);
  Finally
    Refresh;
  End;
end;

procedure TReaderForm.PBLoadClick(Sender: TObject);
begin
//SendIOMsg(ReaderFormHandle,FCLOAD,0,DeckName);
  ReadFirstCard;
  Refresh;
  MainForm.SetButtons;
end;

procedure TReaderForm.ReadFirstCard;
Var AD: Word;
Begin
  For AD:=1 to 80 do
    SetCoreWMCH(AD,False,0);
  SetCoreWM(1,True);
  ReadCard;
  CPUThread.RunState:=RSCLOD;
End;

Procedure TReaderForm.ReadCard;
Var CD: TCard;
    CP: Word;
    CH: Char;
    CV: Byte;
Begin
  FillChar(CD,80,' ');
  If CardsLeft=0 then begin
    If DummyCard<>'' then begin
      CD:=DummyCard;
    End else Begin
      Check:=True;
      CPUThread.RunState:=RSCHEK;
      RaiseIOCheck('Card reader empty',0);
    End;
  End else Begin;
    CD:=ReadDeck[0];
    ReadDeck.Delete(0);
    RDRNDeck.Add(CD);
  End;
  LastWasRead:=True;
  For CP:=1 to 80 do begin
    CH:=CD[CP];
    CV:=ASC2BCD[Ord(CH)];
    SetCoreCH(CP,CV);
  End;
End;

Procedure TReaderForm.PunchCard;
Var CD: TCard;
    AD,CP: Word;
    CH: Byte;
Begin
  AD:=101;
  CD:=StringOfChar(' ',80);
  For CP:=1 to 80 do begin
    CH:=GetCoreCH(AD);
    CD[CP]:=ASC(CH);
    Inc(AD);
  End;
  PUNNDeck.Add(CD);
  LastWasRead:=False;
End;

Procedure TReaderForm.ReStackCard(SN: Byte);
Var CD: TCard;
Begin
  If LastWasRead then With RDRNDeck do begin
    CD:=Strings[Count-1];
    Delete(Count-1);
  End else With PUNNDeck do begin
    CD:=Strings[Count-1];
    Delete(Count-1);
  End;
  Case SN of
    1: RDR1Deck.Add(CD);
    2: RP82Deck.Add(CD);
    4: PUN4Deck.Add(CD);
    8: RP82Deck.Add(CD);
  End;
End;

Initialization
End.
