//******************************************************************************
// This unit implements the functions of the IBM 1403 line printer.
// Other units send Windows messages to this form to invoke the required I/O
// operations.
// Needs improvement to include a carriage control tape...
//******************************************************************************
Unit IBM1403;

Interface

Uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, ExtCtrls, StdCtrls, ComCtrls,
     URecDisp,     // Unit record display component
     IBMTrace,     // Log/Trace display
     IBMChars,     // Character codes and mappings
     BFOOCore,     // IBM 1401 core storage system
     BFOODefs;     // Global, utility, and miscellaneous definitions

Type
  TPrinterForm = class(TForm)
    Panel1: TPanel;
    LALineCount: TLabel;
    Label1: TLabel;
    CBSize: TCheckBox;
    Printer: TUnitRecordDisplay;
    Timer1: TTimer;
    procedure FormResize(Sender: TObject);
    procedure CBSizeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    PrintOut: TStringList;
    SkipCount: Integer;
    procedure Refresh;
    procedure PrintBlankLines(LC: Integer; IP: Boolean);
    procedure PrintLine(PM: Boolean);
  protected
    procedure WMSetSize(Var Message: TMessage); Message WM_SetSize;
    procedure WMIOControl(Var Message: TMessage); Message WM_IOCONTROL;
  public
    Function  LineCount: Word;
  end;

Var PrinterForm: TPrinterForm;

Implementation

{$R *.DFM}

procedure TPrinterForm.FormCreate(Sender: TObject);
begin
  Inherited;
  Caption:='IBM Emulation - 1403 Printer';
  PrintOut:=TStringList.Create;
  Printer.Records:=PrintOut;
  PrinterFormHandle:=Handle;
end;

procedure TPrinterForm.FormDestroy(Sender: TObject);
begin
  Printout.SaveToFile('B1401.PRN');
  PrintOut.Free;
end;

Procedure TPrinterForm.FormResize(Sender: TObject);
begin
  Printer.SetSize; // Sends a WM_SetSize when done
end;

Procedure TPrinterForm.WMSetSize(Var Message: TMessage);
begin
  Inherited;
  Width:=Printer.Width+8;
End;

Procedure TPrinterForm.CBSizeClick(Sender: TObject);
begin
  If Not CBSize.Checked then Printer.Font.Size:=6
                        else Printer.Font.Size:=8;
end;

// Message handler for I/O commands
Procedure TPrinterForm.WMIOControl(Var Message: TMessage);
Var IP: Boolean;
    LC: Word;
begin
  With PIOControl(Message.LParam)^ do 
    Case Action Of
      FCRESET:  PrintOut.Clear;
      FCSAVE:   PrintOut.SaveToFile(StrParm^);
      FCWRITE:  PrintLine(SubParm=1);
      FCCOUNT:  IOCount:=PrintOut.Count;
      FCWRBLNK: Begin
                  LC:=SubParm;
                  IP:=LC>=1000; If IP then Dec(LC,1000);
                  PrintBlankLines(LC,IP);
                End;
      Else      RaiseIOCheck('Invalid I/O Op for PRN: '+IntToStr(Action),0);
    End;
  If Not TurboMode then
    Refresh;
end;

Procedure TPrinterForm.PrintBlankLines(LC: Integer; IP: Boolean);
Var CC: Integer;
Begin
  // Immediate print?
  If Not IP then begin
    SkipCount:=LC;           // Set count for next print, don't print now
  End else Begin
    For CC:=1 to LC do       // Add required blank lines
      PrintOut.Add('');
    SkipCount:=0;            // Reset deferred skip count
  End;
End;

Procedure TPrinterForm.PrintLine(PM: Boolean);
Var WM: Boolean;
    CH: Byte;
    SS: String;
    AD: Word;
Begin
  SS:='';                              // Build print line from core at 201
  For AD:=201 to 332 do begin
    BAReg:=AD;
    GetCoreWMCH(AD,WM,CH);
    If PM then begin                   // With WordMarks?
      If WM then CH:=1 else CH:=0;
    End;
    SS:=SS+PRT(CH);
  End;
{
If Copy(SS,1,6)=' 4 010' then begin
  TraceForm.CBInstTrace.Checked:=True;
End;
If Copy(SS,1,6)=' 4 030' then begin
  TraceForm.CBInstTrace.Checked:=False;
End;
}
  PrintOut.Add(SS);                    // Print the line,
  PrintBlankLines(SkipCount,True);     // Then any pending "skip" lines
  If Not TurboMode then
    Printer.SetTopLine(PrintOut.Count-Printer.DispLines);
End;

procedure TPrinterForm.Refresh;
begin
  Printer.Refresh;
  LALineCount.Caption:=IntToStr(PrintOut.Count);
End;

function TPrinterForm.LineCount: Word;
begin
  Result:=PrintOut.Count;
End;

procedure TPrinterForm.Timer1Timer(Sender: TObject);
begin
  Refresh;
end;

End.
