//******************************************************************************
// This unit implements the functions of the IBM 729 model IV magnetic tape unit.
// Other units send Windows messages to this form to invoke the required I/O
// operations. Buttons allow a tape file to selected for loading, and to cause
// the first record to be read into core storage.
//******************************************************************************
Unit IBM729IV;

Interface

Uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, ExtCtrls,StdCtrls, ComCtrls,
     TapeDisp,     // Tape functions component
     IBMTrace,     // Log/Trace display
     IBMDefs,      // Global, utility, and miscellaneous definitions
     IBMChars,     // Character codes and mappings
     BFOOCore,     // IBM 1401 core storage system
     BFOOProc,     // IBM 1401 Central Processing Unit
     BFOODefs;     // Global, utility, and miscellaneous definitions

Type
  TTapeUnitForm = class(TForm)
    TapeDrive1: TTapeDrive;
    TapeDrive2: TTapeDrive;
    TapeDrive3: TTapeDrive;
    TapeDrive4: TTapeDrive;
    TapeDrive5: TTapeDrive;
    TapeDrive6: TTapeDrive;
    procedure FormCreate(Sender: TObject);
    procedure TapeDriveCoreRead(AD: Word; var WM: Boolean; var CH: Byte);
    procedure TapeDriveCoreWrite(AD: Word; WM: Boolean; CH: Byte);
    procedure TapeDriveLoadClick(Sender: TObject);
  private
  protected
    procedure WMIOControl(Var Message: TMessage); Message WM_IOCONTROL;
    procedure WMZap(Var Message: TMessage);       Message WM_ZAP;
  public
  End;

Var TapeUnitForm: TTapeUnitForm;

Implementation

Uses BFOOMain;     // Main emulator window

{$R *.DFM}

procedure TTapeUnitForm.FormCreate(Sender: TObject);
begin
  Inherited;
  Caption:='IBM Emulation - 729IV Tape Units';
  TapeUnitFormHandle:=Handle;
//  TapeName:='D1401TP.BCD';
//TapeName:='AU1401S.BIN';
//  If ParamCount>0 then TapeName:=ParamStr(1);
end;

procedure TTapeUnitForm.WMIOControl(Var Message: TMessage);
Var TU: TTapeDrive;
begin
  With PIOControl(Message.LParam)^ do begin
    // Select addressed tape unit or process actions directed to all units
    TU:=NIL;
    Case SubParm of
      // Operations on all tape units
      0:   If Action In [FCRESET] then begin
             TapeDrive1.Reset; TapeDrive2.Reset; TapeDrive3.Reset;
             TapeDrive4.Reset; TapeDrive5.Reset; TapeDrive6.Reset;
             Exit;
           End else Begin
             RaiseIOCheck('Invalid I/O Op for All Tape Units: '+IntToStr(Action),0);
           End;
      // Select addressed tape unit
      1:   TU:=TapeDrive1; 2:   TU:=TapeDrive2; 3:   TU:=TapeDrive3;
      4:   TU:=TapeDrive4; 5:   TU:=TapeDrive5; 6:   TU:=TapeDrive6;
      Else Begin
             RaiseIOCheck('Invalid tape unit: '+IntToStr(SubParm),0);
             Exit;
           End;
    End;
    // Perform operation using selected tape unit
    Try
      Case Action Of
        FCOPEN:   TU.MountTape(StrParm^);
        FCLOAD:   Begin
                    TU.ReadRecord(1,XFerCount);
                    TapeDriveLoadClick(Self);
                  End;
        FCREAD:   Begin
                    TU.ReadRecord(BAReg,XFerCount);
                    //AAReg:='%4x';???
                    BAReg:=BAReg+XFerCount;
                  End;
        FCWRITE:  Begin
                    TU.WriteRecord(BAReg,XFerCount);
                    //AAReg:='%4x';???
                    BAReg:=BAReg+XFerCount;
                  End;
        FCRSTIND: TU.ResetTapeIndicate;
        FCPROTON: TU.SetWriteProtect(True);
        FCPROTOF: TU.SetWriteProtect(False);
        FCREWIND: TU.Rewind;
        FCREWUNL: TU.RewindUnload;
        FCWTMARK: TU.WriteTapeMark;
        Else      RaiseIOCheck('Invalid I/O Op for Tape: '+IntToStr(Action),0);
      End;
    Except
      On E: Exception do begin
        StopMesg:=E.Message;
        CPUThread.RunState:=RSCHEK;
      End;
    End;
  End;
  MainForm.SetButtons;
End;

procedure TTapeUnitForm.WMZap(var Message: TMessage);
Var TU: TTapeDrive;
begin
  With PZapMsg(Message.LParam)^ do begin
    // Select addressed tape unit or process actions directed to all units
    TU:=NIL;
    // Select addressed tape unit
    Case Target of
      1:   TU:=TapeDrive1; 2:   TU:=TapeDrive2; 3:   TU:=TapeDrive3;
      4:   TU:=TapeDrive4; 5:   TU:=TapeDrive5; 6:   TU:=TapeDrive6;
      Else Begin
             RaiseIOCheck('Invalid tape unit: '+IntToStr(Target),0);
             Exit;
           End;
    End;
    // Perform operation using selected tape unit
    TU.ZapTapeData(Address,Value);
  End;
end;

procedure TTapeUnitForm.TapeDriveCoreRead(AD: Word; Var WM: Boolean; Var CH: Byte);
begin
  GetCoreWMCH(AD,WM,CH);
end;

procedure TTapeUnitForm.TapeDriveCoreWrite(AD: Word; WM: Boolean; CH: Byte);
begin
  SetCoreWMCH(AD,WM,CH);
end;

procedure TTapeUnitForm.TapeDriveLoadClick(Sender: TObject);
begin
  CPUThread.RunState:=RSTLOD;
  MainForm.SetButtons;
end;

Initialization
End.
