//******************************************************************************
// This unit provides a log/trace display form to aid with debugging.
// Other units send Windows messages to this form using the procedures exposed
// by this unit. The messages contain character strings to be displayed.
// The first character is stripped from the string and used to identify the
// message type and to colour-code it when displayed. Checkboxes control which
// types of messages are shown
//******************************************************************************
Unit IBMTrace;

Interface

Uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
     IBMDefs;      // Global, utility, and miscellaneous definitions

// Trace Control flags
Var TraceDets:  Boolean;          // Execution details
    TraceRegs:  Boolean;          // Register values
    TraceScrs:  Boolean;          // Script execution
    TraceThds:  Boolean;          // Thread status

Type
  TTraceForm = class(TForm)
    LBLog: TListBox;
    Panel1: TPanel;
    CBInstTrace: TCheckBox;
    CBTraceDets: TCheckBox;
    CBTraceRegs: TCheckBox;
    CBTraceScrs: TCheckBox;
    CBTraceThds: TCheckBox;
    PBSave: TButton;
    PBClear: TButton;
    procedure CBTraceClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LBLogDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure PBClearClick(Sender: TObject);
    procedure PBSaveClick(Sender: TObject);
  private
    FInstTrace: Boolean;
    procedure SetInstTrace(Const TM: Boolean);
  protected
    procedure WMTrace(Var Message: TMessage); Message WM_TRACE;
  public
    Procedure Clear;
    Property  InstTrace: Boolean Read FInstTrace Write SetInstTrace;
  end;

Var TraceForm: TTraceForm;

Procedure PutTrace(SS: String);
Procedure PutThrTrace(SS: String);
Procedure PutErrTrace(SS: String);
Procedure PutInsTrace(SS: String);

Implementation

{$R *.DFM}

procedure TTraceForm.FormCreate(Sender: TObject);
begin
  Inherited;
  TraceFormHandle:=Handle;
  PBSave.Hint:='Save list to '+EXEName+'.Log';
end;

procedure TTraceForm.SetInstTrace(Const TM: Boolean);
begin
  CBInstTrace.Checked:=TM;
  FInstTrace:=TM;
end;

procedure TTraceForm.CBTraceClick(Sender: TObject);
begin
  InstTrace:=CBInstTrace.Checked;
  TraceDets:=CBTraceDets.Checked;
  TraceRegs:=CBTraceRegs.Checked;
  TraceThds:=CBTraceThds.Checked;
  TraceScrs:=CBTraceScrs.Checked;
  CBTraceDets.Enabled:=InstTrace;
  CBTraceRegs.Enabled:=InstTrace;
end;

procedure TTraceForm.PBSaveClick(Sender: TObject);
begin
  LBLog.Items.SaveToFile('B1401.LOG');
end;

procedure TTraceForm.PBClearClick(Sender: TObject);
begin
  Clear;
end;

Procedure TTraceForm.Clear;
begin
  LBLog.Clear;
end;

procedure TTraceForm.WMTrace(Var Message: TMessage);
Var TS: String;
begin
  TS:=PString(Message.LParam)^;
  If TS<>'' then
    With LBLog do begin
      Items.Add(TS);
      TopIndex:=Items.Count-1;
      If Items.Count>30000 then Clear;
    End;
End;

Procedure TTraceForm.LBLogDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
Var SS: String;
Begin
  With (Control as TListBox),Canvas do begin
    FillRect(Rect);
Try
    SS:=Items[Index];
Except
End;
    Case SS[1] of
      'I': Font.Color:=CLBlack;         // Instruction fetches
      'D': Font.Color:=CLGray;          // Instruction execution details
      'R': Font.Color:=CLBlue;          // Register values
      'E': Font.Color:=CLRed;           // Error messages
      'S': Font.Color:=$0000CA65;       // Script action
      'T': Font.Color:=CLGreen;         // Thread messages
      Else Font.Color:=$004080FF;
    End;
    Delete(SS,1,1);
    TextOut(Rect.Left,Rect.Top,SS);
  End;
End;

Procedure PutTrace(SS: String);
Begin
  SendMessage(TraceFormHandle,WM_Trace,0,Longint(@SS));
End;

Procedure PutThrTrace(SS: String);
Begin
  If TraceThds then begin
    SS:='T'+SS;
    SendMessage(TraceFormHandle,WM_Trace,0,Longint(@SS));
  End;
End;

Procedure PutErrTrace(SS: String);
Begin
  SS:='E'+SS;
  SendMessage(TraceFormHandle,WM_Trace,0,Longint(@SS));
End;

Procedure PutInsTrace(SS: String);
Begin
  SS:='I'+SS;
  SendMessage(TraceFormHandle,WM_Trace,0,Longint(@SS));
End;

End.

