//******************************************************************************
// This unit implements a Delphi component to display a list of unit records -
// (card images and print lines).
// It exists mainly to allow card decks to be displayed in reverse order with
// first card at the bottom of the list. But it does provide other functions
// that are useful for both unit record types.
// It still needs some work done on the scrollbar behaviour...
//******************************************************************************
Unit URecDisp;

Interface

Uses Windows,Messages,SysUtils,Classes,StdCtrls,
     Graphics,Controls,Forms,Dialogs;

Const WM_SETSIZE    =WM_APP+300;

Type TCard=String[80];
     TLine=String[132];

Type TURMode=(URMRDR,URMPUN,URMPRN);

Type TUnitRecordDisplay = class(TCustomControl)
        constructor Create(AOwner: TComponent);            Override;
        destructor Destroy;                                Override;
        Procedure Loaded;                                  Override;
      private
        SBVert:    TScrollBar;
        DrwBitMap: TBitMap;   // Offscreen bitmap (to avoid flicker)
        DrwCanvas: TCanvas;   // And its canvas
        CharWid,
        CharHgt:  Integer;
        LftChar:  Integer;
        TopRec:   Integer;
        BotRec:   Integer;
        FURMode:  TURMode;
        SetOrLock: Boolean;
        FRecords: TStringList;
        ParentWindow: HWnd;
        procedure SBVertChange(Sender: TObject);
        procedure FontChanged(Sender: TObject);
        procedure SetDisplayOrigin(X, Y: Integer);
        procedure SetURMode(const Value: TURMode);
        procedure SetVSBMax;
        procedure SetRecords(const Value: TStringList);
      protected
        Procedure WMERASEBKGND(Var WM: TWMERASEBKGND);     Message WM_ERASEBKGND;
        procedure Paint;                                   Override;
      public
        RecSize:   Integer;
        DispLines: Integer;
        procedure SetSize;
        Procedure Refresh;
        Procedure SetTopLine(LN: Integer);
      published
        Property URMode: TURMode Read FURMode Write SetURMode;
        Property Records: TStringList Read FRecords Write SetRecords;
        Property Align;
        Property Font;
      end;

procedure Register;

implementation

Const SBSize=18;

//******************************************************************************
// Component Registration procedure, Create, Destroy and Component loaded methods
//******************************************************************************
procedure Register;
begin
  RegisterComponents('EmulComps', [TUnitRecordDisplay]);
end;

Constructor TUnitRecordDisplay.Create(AOwner: TComponent);
Begin
  Inherited;
  Name:='UnitRecord';
  URMode:=URMRDR; RecSize:=80;
  TabStop:=True;
  Color:=CLWhite;
  Width:=100; Height:=100;
  LftChar:=1; BotRec:=1;
  With Font do begin
    Name:='lucidia console'; Size:=7; Pitch:=FPFixed;
  End;
//OnClick:=MOClick;
//OnDblClick:=DBClick;
  DrwBitMap:=TBitMap.Create;
  SBVert:=TScrollBar.Create(Self);
  With SBVert do begin
    Parent:=Self;       TabStop:=False;
    Kind:=SBVertical;   Align:=ALRight;
    Width:=SBSize;      OnChange:=SBVertChange;
  End;
  Font.OnChange:=FontChanged;
End;

destructor TUnitRecordDisplay.Destroy;
begin
  SBVert.Free;
  DrwBitMap.Free;
  Inherited;
end;

procedure TUnitRecordDisplay.Loaded;
begin
  Inherited;
  If Parent<>NIL then
    ParentWindow:=Parent.Handle;
End;

procedure TUnitRecordDisplay.SetRecords(const Value: TStringList);
begin
  FRecords := Value;
end;

procedure TUnitRecordDisplay.SetURMode(const Value: TURMode);
begin
  FURMode:=Value;
  If FRecords<>NIL then FRecords.Clear;
  If Value=URMPRN then RecSize:=132 else RecSize:=80;
  SetSize;
end;

procedure TUnitRecordDisplay.FontChanged(Sender: TObject);
begin
  SetSize;
end;

Procedure TUnitRecordDisplay.SetSize;
Var CH: Integer;
Begin
  If DrwBitMap=NIL then Exit;
  With DrwBitMap do begin
    With Canvas do begin
      Font.Assign(Self.Font);             // Set to same font as screen
      With TextExtent('WM') do begin
        CharWid:=CX Div 2; CharHgt:=CY;
      End;
    End;
    Width:=(CharWid*RecSize)+2;
    Self.Width:=Width+SBSize;
    Height:=Self.Height;
    CH:=Height Div CharHgt;
    DrwCanvas:=Canvas;                  // Make it available for drawing on
  End;
  With SBVert do begin
    SmallChange:=1;
    LargeChange:=CH;
    Min:=1; Max:=CH;
  End;
  SetDisplayOrigin(1,1);
  PostMessage(ParentWindow,WM_SetSize,0,0);
  DispLines:=CH;
end;

procedure TUnitRecordDisplay.WMERASEBKGND(Var WM: TWMERASEBKGND);
begin
  WM.Result:=1;
end;

procedure TUnitRecordDisplay.Paint;
Var PY: Integer;
    RN: Word;
    RS: String[132];
begin
  Inherited;
  If CSLoading in ComponentState then Exit;
  If DrwBitMap=NIL then Exit;
  // Work with offscreen bitmap's canvas
  With DrwCanvas do begin
    // Clear it
    Brush.Color:=Self.Color;
    FillRect(Rect(0,0,Width,Height));
    Pen.Style:=PSSolid; Pen.Color:=CLBlack;
    If FRecords<>NIL then
      If FURMode=URMRDR then begin
        RN:=BotRec;
        PY:=Height-CharHgt;
        While (PY>0) And (RN<=FRecords.Count) do begin
          RS:=FRecords[RN-1];
          RS:=Copy(RS,LftChar,RecSize);
          TextOut(0,PY,RS);
          Dec(PY,CharHgt); Inc(RN);
        End;
      End else Begin
        RN:=TopRec;
        If RN>FRecords.Count then RN:=FRecords.Count;
        PY:=0;
        While (PY<=Height) And (RN<=FRecords.Count) And (RN>0) do begin
          RS:=FRecords[RN-1];
          RS:=Copy(RS,LftChar,RecSize);
          TextOut(0,PY,RS);
          Inc(PY,CharHgt); Inc(RN);
        End;
      End;
  End;
  // Update screen canvas from offscreen canvas
  Canvas.Draw(0,0,DrwBitMap);
End;

procedure TUnitRecordDisplay.SBVertChange(Sender: TObject);
begin
  If FURMode=URMRDR then
    SetDisplayOrigin(LftChar,SBVert.Max-SBVert.Position+1)
  Else
    SetDisplayOrigin(LftChar,SBVert.Position);
end;

procedure TUnitRecordDisplay.SetVSBMax;
begin
  If FRecords=NIL then Exit;
  If FRecords.Count=0 then SBVert.Max:=1
                      else SBVert.Max:=FRecords.Count;
End;

procedure TUnitRecordDisplay.SetDisplayOrigin(X,Y: Integer);
begin
  If FRecords=NIL then Exit;
  If SetOrLock then Exit;
  SetOrLock:=True;
  LftChar:=X;
  SetVSBMax;
  If Y>FRecords.Count then Y:=FRecords.Count;
  If Y<1 then Y:=1;
  If FURMode=URMRDR then begin
    BotRec:=Y;
    SBVert.Position:=SBVert.Max-Y+1;
  End else Begin
    TopRec:=Y;
    SBVert.Position:=Y;
  End;
  Invalidate;
  SetOrLock:=False;
end;

procedure TUnitRecordDisplay.Refresh;
begin
  SetVSBMax;
  Invalidate;
end;

procedure TUnitRecordDisplay.SetTopLine(LN: Integer);
begin
  SBVert.Position:=LN;
end;

end.
